#!/bin/sh
# ZipitZ2 Stock (blob+2.6.21.1) to u-boot flasher script v0.02 
# Copyright (C) 2010 Marek Vasut <marek.vasut@gmail.com>
# This is under GPL v3
# Modified by mozzwald http://www.mozzwald.com
# Modified by rkdavis  http://russelldavis.org

# filenames
image=image.img
partlist=plist.lst
flashdefs=flash.def
restoredir=`/mnt/sd0/bin/busybox date +%m%d%Y_%H%M%S`

# first warning!
dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "WARNING: THIS COULD BRICK YOUR ZIPIT! \
	No one but YOU is responsible if something goes wrong." $DIALOGSIZE

dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Preparing flash image.." $DIALOGSIZE
if [ -e "$image" ] ; then
	rm -f "$image";
fi
touch "$image"

if [ -e "$partlist" ] ; then
	rm -f "$partlist";
fi
touch "$partlist"

# create the complete image file with proper spacing
offset=0
while read file start md5; do
	if ! [ -e "$file" ] ; then
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox  "File \"$file\" not found, aborting!" $DIALOGSIZE
		exit 1;
	fi

	# check md5sum
	if [ -z "$md5" ] ; then
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox  "md5sum for \"$file\" not found, aborting!" $DIALOGSIZE
		exit 2;
	elif [ "$md5" != "`/mnt/sd0/bin/busybox md5sum $file | /mnt/sd0/bin/busybox cut -d " " -f 1`" ] ; then
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox "md5sum for \"$file\" does not match, aborting!" $DIALOGSIZE
		exit 3;
	fi

	# calculate empty space between parts
	pad=`expr "$start" - "$offset"`

	if [ "$pad" -lt 0 ] ; then
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Image components overlap, aborting!" $DIALOGSIZE
		exit 2;
	elif [ "$pad" -gt 0 ] ; then
		# write empty space to image file
		dd if=/dev/zero bs="$pad" count=1 >> "$image" 2>/dev/null;
	fi

	# write part to image file
	cat "$file" >> "$image"
	# calculate next offset
	offset=`wc -c "$image" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`;
	#echo "Offset: $offset"
	md5="\0"
done < "$flashdefs"

dialog $BEGIN --backtitle "$BACKTITLE" --infobox "$image ready for flashing." $DIALOGSIZE

# make restore directory
mkdir "$restoredir"

# fix mtdblock partitions, create partition list and backup old flash
flashsz=0
#for i in `/mnt/sd0/bin/busybox ls -1 /dev/ | /mnt/sd0/bin/busybox egrep "mtdblock[[:digit:]]+"` ; do
while read line; do
        i=`echo $line | grep mtdblock | cut -d " " -f 4`
        if [ -n "$i" ]; then
		# fix the mtdblock
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Fixing /dev/$i" $DIALOGSIZE
		blocknum=`echo "$i" | sed -e "s/^.*\(.\)$/\1/"`
		#blocknum=${i#/dev/mtdblock}
		rm "/dev/$i"
		mknod "/dev/$i" b 31 "$blocknum"
		# write the partition list
		partsz=`wc -c "/dev/$i" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`
		echo "/dev/$i $partsz" >> "$partlist"
		# Backup Partitions & Create restore.def
		dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Backing up /dev/$i" $DIALOGSIZE
		dd if="/dev/$i" of="$restoredir/$i.bak"
	        mtdmd5=`md5sum "$restoredir/$i.bak" | cut -d " " -f 1`
	        echo "$i.bak $flashsz $mtdmd5" >> "$restoredir"/restore.def
		flashsz=`expr "$flashsz" + "$partsz"`
		mtdcount=`expr "$blocknum" + 1`
	fi
done </proc/partitions

if [ "$offset" -gt "$flashsz" ] ; then
	dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Image too big for flash, aborting! Offset: $offset FlashSize: $flashsz" $DIALOGSIZE
	sleep 2
	exit 3;
fi

if [ -z "$partlist" ] ; then
	dialog $BEGIN --backtitle "$BACKTITLE" --infobox  "Partition list empty, aborting!" $DIALOGSIZE
	sleep 2
	exit 4;
fi

dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "Make sure your Zipit is fully charged and you have the AC Adapter plugged in. " $DIALOGSIZE

dialog $BEGIN --backtitle "$BACKTITLE" --no-cancel --inputbox "Last chance before flashing. Are you sure (yes/no)" $DIALOGSIZE 2>/tmp/nogoingback

flash=`cat /tmp/nogoingback`
if [ "$flash" != "yes" ]; then
	dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "You didn't enter yes so aborting" $DIALOGSIZE
	exit 5
fi


dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Partitions to be over-written: $mtdcount Starting flashing procedure, do NOT POWER OFF!" $DIALOGSIZE

offset=0
while read part size ; do
	dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Writing to $part" $DIALOGSIZE
	dd if="$image" of="$part" bs=1 skip="$offset" count="$size"
	offset=`expr "$offset" + "$size"`;
done < "$partlist"

rm -f "$partlist" "$image"

dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "Flashing complete!" $DIALOGSIZE

dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "After the system is halted, unplug the AC Power AND remove the battery for a moment.\
	After you re-attach the battery/AC, the system should boot." $DIALOGSIZE

